(* PRACTICA 5. IMPLEMENTACION DE UN SISTEMNA DE RESERVAS DE VUELOS
          PARA UN AEROPUERTO MEDIANTE UNA LISTA DE LISTAS *)

IMPLEMENTATION MODULE Vuelo;

FROM Storage IMPORT ALLOCATE,DEALLOCATE,Available;
FROM IO IMPORT WrStr,WrLn,WrCard,RdKey;
FROM Window IMPORT Clear;

VAR
  error:ERROR;



PROCEDURE Crear(VAR aeropuerto:TipoVuelo);
   (* Notar que no hay posibilidad de error. El tipo estransparente ademas *)
BEGIN
   aeropuerto:=NIL;
END Crear;

(* Procedimiento local *)
PROCEDURE Longitud(a:ARRAY OF CHAR):CARDINAL;
(* devuelve la longitud de una cadena *)
VAR
  i:CARDINAL;
BEGIN
  i:=0;
  WHILE (i<=HIGH(a)) AND (a[i]#0C) DO
    INC(i);
  END;
  RETURN i;
END Longitud;

(* Procedimiento local para comparacion de cadenas *)
PROCEDURE IgualC(a,b:ARRAY OF CHAR):BOOLEAN;
(* me compara dos cadenas diciendome si son iguales o no *)
VAR
  i:CARDINAL;
BEGIN
  IF Longitud(a)#Longitud(b) THEN
    RETURN FALSE
  ELSE
    i:=0;
    WHILE (i<=(Longitud(a)-1))AND(CAP(a[i])=CAP(b[i])) DO
      INC(i);
    END;
    RETURN i>(Longitud(a)-1);
  END;
END IgualC;

(* Procedimiento local para verificar si el vuelo existe *)
PROCEDURE EstaVuelo(aeropuerto:TipoVuelo; entrada:ARRAY OF CHAR;
                    VAR ant,pos:TipoVuelo):BOOLEAN;

(* me dice si esta el vuelo, en la posicion que esta y el anterior para
   que me sirva el mismo procedimiento para insertar y borrar *)
BEGIN
  ant:=NIL;
  pos:=aeropuerto;
  WHILE (pos#NIL) AND (NOT(IgualC(pos^.datos.numvuelo,entrada))) DO
    ant:=pos;
    pos:=pos^.sig;
  END;
  RETURN pos#NIL
END EstaVuelo;

PROCEDURE InsertaVuelo(VAR aeropuerto:TipoVuelo; entrada:DatosVuelo;VAR error:ERROR);
VAR
  aux,pos,basura:TipoVuelo;
BEGIN
  error:=SinError;
  IF EstaVuelo(aeropuerto,entrada.numvuelo,basura,basura) THEN
     error:=YaTaVuelo;  (* no hay vuelos repetidos *)
  ELSIF NOT(Available(SIZE(aux^))) THEN  (*Tambien valido SIZE(VUELOS) *)
     error:=SinMemo;   (* No hay memoria para crear un nodo *)
  ELSE
     NEW(aux);
     aux^.datos:=entrada;
     aux^.sig:=NIL;
     aux^.datos.pasajeros:=NIL;
          (* Por simplicidad inserto el nuevo vuelo al principio *)
     aux^.sig:=aeropuerto;
     aeropuerto:=aux;
  END;
END InsertaVuelo;

PROCEDURE EstaPersona(Lpersona:TipoPasajero; entrada:ARRAY OF CHAR;
                      VAR ant,pos:TipoPasajero):BOOLEAN;

(* me dice si esta la persona |con el DNI|, en la posicion que esta y el anterior para
   que me sirva el mismo procedimiento para insertar y borrar.
   *)
BEGIN
  ant:=NIL;
  pos:=Lpersona;
  WHILE (pos#NIL)AND(NOT(IgualC(pos^.valor.dni,entrada))) DO
    ant:=pos;
    pos:=pos^.sig;
  END;
  RETURN pos#NIL
END EstaPersona;

PROCEDURE InsertaPersona(VAR aeropuerto:TipoVuelo; entrada:DatosPersona;
                         n_vuelo:ARRAY OF CHAR; VAR error:ERROR);
(*  Si hay algun error devuelve el correspondiente valor del enumerado *)
VAR
  basura,pos:TipoVuelo;
  aux:TipoPasajero;
  basura2,pasaje:TipoPasajero;
BEGIN
  error:=SinError;
  IF NOT(EstaVuelo(aeropuerto,n_vuelo,basura,pos)) THEN
     error:=NoVuelo;   (* El vuelo no existe *)
  ELSIF pos^.datos.libres=0 THEN
     error:=Vlleno; (*el vuelo esta lleno*)
  ELSIF EstaPersona(pos^.datos.pasajeros,entrada.dni,basura2,basura2) THEN
     error:=YaTaPersona;
  ELSIF NOT(Available(SIZE(aux^))) THEN
     error:=SinMemo;
  ELSE
     NEW(aux);
     aux^.valor:=entrada;

     DEC(pos^.datos.libres);
     pasaje:=pos^.datos.pasajeros;
        (* Por simplicidad inserto siempre por el principio ya que
           no se me indica que las listas deban estar ordenadas *)
     aux^.sig:=pos^.datos.pasajeros;
     pos^.datos.pasajeros:=aux
   END;
END InsertaPersona;

(* Procedimiento local al modulo. Para el formateo de la pantalla *)
PROCEDURE Pantalla(VAR i:CARDINAL);
          VAR
            basura:CHAR;
          BEGIN
              IF i>22 THEN
                 WrLn;WrStr('Pulsa una tecla para continuar ');
                 basura:=RdKey();
                 i:=0;
                 Clear;
              END;
END Pantalla;


PROCEDURE Visualizar(aeropuerto:TipoVuelo);
VAR
  aux:TipoVuelo;
  pos:TipoPasajero;
  i:CARDINAL;
BEGIN
     (* No hay posibilidad de error. Si en el aeropuerto no hay
        vuelos entonces no visualizara nada. Es un estado (vacio)
        no un error *)
  aux:=aeropuerto;
  i:=9; (* por los nueve iniciales del menu*)
  WHILE aux#NIL DO
     WrLn;
     INC(i);
     Pantalla(i);
     WrStr('el vuelo es : ');
     WrStr(aux^.datos.numvuelo);
     WrStr('  plazas : ');
     WrCard(aux^.datos.libres,0);WrLn;
     INC(i);
     Pantalla(i);
     WrStr('pasajeros ');WrLn;
     INC(i);
     Pantalla(i);
     pos:=aux^.datos.pasajeros;
     WHILE pos#NIL DO
        WrStr(pos^.valor.nombre);WrStr('  ');
        WrStr(pos^.valor.dni);WrStr('  ');
        WrCard(pos^.valor.edad,0);WrLn;
        INC(i);
        Pantalla(i);
        pos:=pos^.sig;
     END;
     aux:=aux^.sig;
   END;
END Visualizar;

PROCEDURE VisualizarVuelo(aeropuerto:TipoVuelo; n_vuelo:ARRAY OF CHAR;VAR error:ERROR);
VAR
  pos,basura:TipoVuelo;
  aux:TipoPasajero;
  i:CARDINAL;
BEGIN
  error:=SinError;
  IF NOT(EstaVuelo(aeropuerto,n_vuelo,basura,pos)) THEN
     error:=NoVuelo;
  ELSE
     i:=9;  (*por los 9 iniciales de menu*)
     WrLn;
     INC(i);
     Pantalla(i);
     WrStr('el vuelo es : ');
     WrStr(pos^.datos.numvuelo);
     WrStr('  plazas : ');
     WrCard(pos^.datos.libres,0);WrLn;
     INC(i);
     Pantalla(i);
     WrStr('pasajeros ');WrLn;
     INC(i);
     Pantalla(i);
     aux:=pos^.datos.pasajeros;
     WHILE aux#NIL DO
        WrStr(aux^.valor.nombre);WrStr('  ');
        WrStr(aux^.valor.dni);WrStr('  ');
        WrCard(aux^.valor.edad,0);WrLn;
        INC(i);
        Pantalla(i);
        aux:=aux^.sig;
     END;
  END;
END VisualizarVuelo;

PROCEDURE Destruir(VAR aeropuerto:TipoVuelo);
VAR
  aux:TipoVuelo;
  pos:TipoPasajero;
BEGIN
   (* Tampoco habra error. Si esta vacio entonces no puede destruirlo.
      El estado vacio no se puede diferenciar del no-creado en este caso *)

  WHILE aeropuerto#NIL DO
     aux:=aeropuerto;
     aeropuerto:=aeropuerto^.sig;
     WHILE aux^.datos.pasajeros#NIL DO
       pos:=aux^.datos.pasajeros;
       aux^.datos.pasajeros:=aux^.datos.pasajeros^.sig;
       DISPOSE(pos);
     END;
     DISPOSE(aux);
  END;
END Destruir;

PROCEDURE EliminarP(VAR aeropuerto:TipoVuelo; entrada:ARRAY OF CHAR ;
                    n_vuelo:ARRAY OF CHAR;VAR error:ERROR);
VAR
  basura,pos:TipoVuelo;  (*pos = posicion del vuelo*)
  ant,pasaje:TipoPasajero;
BEGIN
  error:=SinError;
  IF NOT(EstaVuelo(aeropuerto,n_vuelo,basura,pos)) THEN
     error:=NoVuelo;
  ELSIF pos^.datos.pasajeros=NIL THEN
     error:=Vacio; (*el vuelo esta vacio*)
  ELSIF NOT(EstaPersona(pos^.datos.pasajeros,entrada,ant,pasaje)) THEN
     error:=NoPersona;
  ELSE
     IF ant=NIL THEN
        pos^.datos.pasajeros:=pasaje^.sig;
     ELSE
        ant^.sig:=pasaje^.sig;
     END;
     DISPOSE(pasaje);
     INC(pos^.datos.libres)
  END;
END EliminarP;

PROCEDURE EliminarV(VAR aeropuerto:TipoVuelo; entrada:ARRAY OF CHAR; VAR error:ERROR);
VAR
  ant,pos:TipoVuelo;
  aux:TipoPasajero;
BEGIN
  error:=SinError;
  IF NOT(EstaVuelo(aeropuerto,entrada,ant,pos)) THEN
    error:=NoVuelo;  (* no esta el vuelo *)
  ELSE
    WHILE pos^.datos.pasajeros#NIL DO
      aux:=pos^.datos.pasajeros;
      pos^.datos.pasajeros:=pos^.datos.pasajeros^.sig;
      DISPOSE(aux);     (* Libera la memoria para cada pasajero *)
    END;
    IF ant=NIL THEN
       aeropuerto:=aeropuerto^.sig;
    ELSE
       ant^.sig:=pos^.sig;
    END;
    DISPOSE(pos);     (* Libera la memoria para el vuelo *)
  END;
END EliminarV;

END Vuelo.